# CELERY
# Default broker URL. See http://celery.readthedocs.org/en/latest/configuration.html#broker-url.
BROKER_URL = None

# Backend used to store task results.
# See http://celery.readthedocs.org/en/latest/configuration.html#celery-result-backend.
CELERY_RESULT_BACKEND = None

# A sequence of modules to import when the worker starts.
# See http://celery.readthedocs.org/en/latest/configuration.html#celery-imports.
CELERY_IMPORTS = (
    'ecommerce_worker.fulfillment.v1.tasks',
)
# END CELERY


# ORDER FULFILLMENT
# Absolute URL used to construct API calls against the ecommerce service.
ECOMMERCE_API_ROOT = None

# Long-lived access token used by Celery workers to authenticate against the ecommerce service.
WORKER_ACCESS_TOKEN = None

# Maximum number of retries before giving up on the fulfillment of an order.
# For reference, 11 retries with exponential backoff yields a maximum waiting
# time of 2047 seconds (about 30 minutes). Defaulting this to None could yield
# unwanted behavior: infinite retries.
MAX_FULFILLMENT_RETRIES = 11
# END ORDER FULFILLMENT



import os
from celery import Celery


# Environment variable indicating which configuration module to use.
CONFIGURATION = 'WORKER_CONFIGURATION_MODULE'

# Set the default configuration module, if one is not aleady defined.
os.environ.setdefault(CONFIGURATION, 'ecommerce_worker.configuration.local')

app = Celery('ecommerce_worker')
# See http://celery.readthedocs.org/en/latest/userguide/application.html#config-from-envvar.
app.config_from_envvar(CONFIGURATION)



"""Order fulfillment tasks."""
from celery import shared_task
from celery.exceptions import Ignore
from celery.utils.log import get_task_logger
from ecommerce_api_client import exceptions
from ecommerce_api_client.client import EcommerceApiClient

from ecommerce_worker.utils import get_configuration

logger = get_task_logger(__name__)  # pylint: disable=invalid-name


@shared_task(bind=True, ignore_result=True)
def fulfill_order(self, order_number):
    """Fulfills an order.

    Arguments:
        order_number (str): Order number indicating which order to fulfill.

    Returns:
        None
    """
    ecommerce_api_root = get_configuration('ECOMMERCE_API_ROOT')
    worker_access_token = get_configuration('WORKER_ACCESS_TOKEN')
    max_fulfillment_retries = get_configuration('MAX_FULFILLMENT_RETRIES')

    if not (ecommerce_api_root and worker_access_token and max_fulfillment_retries):
        raise RuntimeError('Worker is improperly configured for order fulfillment.')

    api = EcommerceApiClient(ecommerce_api_root, oauth_access_token=worker_access_token)

    try:
        logger.info('Requesting fulfillment of order [%s].', order_number)
        api.orders(order_number).fulfill.put()
    except exceptions.HttpClientError as exc:
        status_code = exc.response.status_code
        if status_code == 406:
            # The order is not fulfillable. Therefore, it must be complete.
            logger.info('Order [%s] has already been fulfilled. Ignoring.', order_number)
            raise Ignore()
        else:
            # Unknown client error. Re-raise the exception.
            logger.exception('Fulfillment of order [%s] failed.', order_number)
            raise exc
    except (exceptions.HttpServerError, exceptions.Timeout) as exc:
        # Fulfillment failed. Retry with exponential backoff until fulfillment
        # succeeds or the retry limit is reached. If the retry limit is exceeded,
        # the exception is re-raised.
        retries = self.request.retries
        if retries == max_fulfillment_retries:
            logger.exception('Fulfillment of order [%s] failed.', order_number)
        else:
            logger.exception('Fulfillment of order [%s] failed. Retrying.', order_number)

        countdown = 2 ** retries
        raise self.retry(exc=exc, countdown=countdown, max_retries=max_fulfillment_retries)



from logging.config import dictConfig
import os

import yaml

from ecommerce_worker.configuration.base import *
from ecommerce_worker.configuration.logger import get_logger_config


def get_overrides_filename(variable):
    """
    Get the name of the file containing configuration overrides
    from the provided environment variable.
    """
    filename = os.environ.get(variable)

    if filename is None:
        msg = 'Please set the {} environment variable.'.format(variable)
        raise EnvironmentError(msg)

    return filename


# LOGGING
logger_config = get_logger_config()
dictConfig(logger_config)
# END LOGGING


filename = get_overrides_filename('ECOMMERCE_WORKER_CFG')
with open(filename) as f:
    config_from_yaml = yaml.load(f)

# Override base configuration with values from disk.
vars().update(config_from_yaml)



from setuptools import setup, find_packages


with open('README.rst') as readme:
    long_description = readme.read()

setup(
    name='edx-ecommerce-worker',
    version='0.1.0',
    description='Celery tasks supporting the operations of edX\'s ecommerce service',
    long_description=long_description,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet',
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
    ],
    keywords='edx ecommerce worker',
    url='https://github.com/edx/ecommerce-worker',
    author='edX',
    author_email='oscm@edx.org',
    license='AGPL',
    packages=find_packages(exclude=['*.tests']),
    install_requires=['celery', 'edx-ecommerce-api-client'],
)



"""Helper functions."""
import os
import sys

from ecommerce_worker.celery_app import CONFIGURATION


def get_configuration(variable):
    """Get a value from configuration.

    Retrieves the value corresponding to the given variable from the
    configuration module currently in use by the app.

    Arguments:
        variable (str): The name of a variable from the configuration module.

    Returns:
        The value corresponding to the variable, or None if the variable is not found.
    """
    name = os.environ.get(CONFIGURATION)

    # __import__ performs a full import, but only returns the top-level
    # package, not the targeted module. sys.modules is a dictionary
    # mapping module names to loaded modules.
    __import__(name)
    module = sys.modules[name]

    return getattr(module, variable, None)
